"""
Constants for enterprise data app.
"""

ANALYTICS_API_VERSION_ATTR = 'API_VERSION'
ANALYTICS_API_VERSION_0 = 'V0'
ANALYTICS_API_VERSION_1 = 'V1'

UUID4_REGEX = '[0-9a-f]{8}-?[0-9a-f]{4}-?4[0-9a-f]{3}-?[89ab][0-9a-f]{3}-?[0-9a-f]{12}'
